import glob
import os
import tarfile

from . import D2to1TestCase


VERSION = '0.1.dev'


class TestCore(D2to1TestCase):
    def test_setup_py_version(self):
        """
        Test that the `./setup.py --version` command returns the correct
        value without balking.
        """

        self.run_setup('egg_info')
        stdout, _, _ = self.run_setup('--version')
        assert stdout == VERSION

    def test_setup_py_keywords(self):
        """
        Test that the `./setup.py --keywords` command returns the correct
        value without balking.
        """

        self.run_setup('egg_info')
        stdout, _, _ = self.run_setup('--keywords')
        assert stdout == 'packaging,distutils,setuptools'

    def test_sdist_extra_files(self):
        """
        Test that the extra files are correctly added.
        """

        stdout, _, return_code = self.run_setup('sdist', '--formats=gztar')

        # There can be only one
        try:
            tf_path = glob.glob(os.path.join('dist', '*.tar.gz'))[0]
        except IndexError:
            assert False, 'source dist not found'

        tf = tarfile.open(tf_path)
        names = ['/'.join(p.split('/')[1:]) for p in tf.getnames()]

        assert 'extra-file.txt' in names



from __future__ import with_statement

import os
import textwrap

from . import D2to1TestCase
from .util import open_config


class TestHooks(D2to1TestCase):
    def setup(self):
        super(TestHooks, self).setup()
        with open_config(os.path.join(self.package_dir, 'setup.cfg')) as cfg:
            cfg.set('global', 'setup-hooks',
                    'd2to1_testpackage._setup_hooks.test_hook_1\n'
                    'd2to1_testpackage._setup_hooks.test_hook_2')
            cfg.set('build_ext', 'pre-hook.test_pre_hook',
                    'd2to1_testpackage._setup_hooks.test_pre_hook')
            cfg.set('build_ext', 'post-hook.test_post_hook',
                    'd2to1_testpackage._setup_hooks.test_post_hook')

    def test_global_setup_hooks(self):
        """
        Test that setup_hooks listed in the [global] section of setup.cfg are
        executed in order.
        """

        stdout, _, return_code = self.run_setup('egg_info')
        assert 'test_hook_1\ntest_hook_2' in stdout
        assert return_code == 0

    def test_command_hooks(self):
        """
        Simple test that the appropriate command hooks run at the
        beginning/end of the appropriate command.
        """

        stdout, _, return_code = self.run_setup('egg_info')
        assert 'build_ext pre-hook' not in stdout
        assert 'build_ext post-hook' not in stdout
        assert return_code == 0

        stdout, _, return_code = self.run_setup('build_ext')
        assert textwrap.dedent("""
            running build_ext
            running pre_hook d2to1_testpackage._setup_hooks.test_pre_hook for command build_ext
            build_ext pre-hook
        """) in stdout
        assert stdout.endswith('build_ext post-hook')
        assert return_code == 0





from __future__ import with_statement
import os
import shutil
import subprocess
import sys
import tempfile

import pkg_resources

from .util import rmtree, open_config


D2TO1_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__),
                                         os.pardir, os.pardir))


def fake_d2to1_dist():
    # Fake a d2to1 distribution from the d2to1 package that these tests reside
    # in and make sure it's active on the path with the appropriate entry
    # points installed

    class _FakeProvider(pkg_resources.EmptyProvider):
        """A fake metadata provider that does almost nothing except to return
        entry point metadata.
        """

        def has_metadata(self, name):
            return name == 'entry_points.txt'

        def get_metadata(self, name):
            if name == 'entry_points.txt':
                return '[distutils.setup_keywords]\nd2to1 = d2to1.core:d2to1\n'
            else:
                return ''


    sys.path.insert(0, D2TO1_DIR)
    if 'd2to1' in sys.modules:
        del sys.modules['d2to1']
    if 'd2to1' in pkg_resources.working_set.by_key:
        del pkg_resources.working_set.by_key['d2to1']
    dist = pkg_resources.Distribution(location=D2TO1_DIR, project_name='d2to1',
                                      metadata=_FakeProvider())
    pkg_resources.working_set.add(dist)


class D2to1TestCase(object):
    def setup(self):
        self.temp_dir = tempfile.mkdtemp(prefix='d2to1-test-')
        self.package_dir = os.path.join(self.temp_dir, 'testpackage')
        shutil.copytree(os.path.join(os.path.dirname(__file__), 'testpackage'),
                        self.package_dir)
        self.oldcwd = os.getcwd()
        os.chdir(self.package_dir)

    def teardown(self):
        os.chdir(self.oldcwd)
        # Remove d2to1.testpackage from sys.modules so that it can be freshly
        # re-imported by the next test
        for k in list(sys.modules):
            if (k == 'd2to1_testpackage' or
                k.startswith('d2to1_testpackage.')):
                del sys.modules[k]
        rmtree(self.temp_dir)

    def run_setup(self, *args):
        cmd = ('-c',
               'import sys;sys.path.insert(0, %r);'
               'from d2to1.tests import fake_d2to1_dist;'
               'from d2to1.extern.six import exec_;'
               'fake_d2to1_dist();exec_(open("setup.py").read())' % D2TO1_DIR)
        return self._run_cmd(sys.executable, cmd + args)

    def run_svn(self, *args):
        return self._run_cmd('svn', args)

    def _run_cmd(self, cmd, args):
        """
        Runs a command, with the given argument list, in the root of the test
        working copy--returns the stdout and stderr streams and the exit code
        from the subprocess.
        """

        os.chdir(self.package_dir)
        p = subprocess.Popen([cmd] + list(args), stdout=subprocess.PIPE,
                             stderr=subprocess.PIPE)

        streams = tuple(s.decode('latin1').strip() for s in p.communicate())
        print(streams)
        return (streams) + (p.returncode,)



from __future__ import with_statement

import contextlib
import os
import shutil
import stat


from ..extern.six import moves as m
ConfigParser = m.configparser.ConfigParser


@contextlib.contextmanager
def open_config(filename):
    cfg = ConfigParser()
    cfg.read(filename)
    yield cfg
    with open(filename, 'w') as fp:
        cfg.write(fp)


def rmtree(path):
    """
    shutil.rmtree() with error handler for 'access denied' from trying to
    delete read-only files.
    """

    def onerror(func, path, exc_info):
        if not os.access(path, os.W_OK):
            os.chmod(path, stat.S_IWUSR)
            func(path)
        else:
            raise

    return shutil.rmtree(path, onerror=onerror)



#!/usr/bin/env python
try:
    from setuptools import setup
except ImportError:
    from distribute_setup import use_setuptools
    use_setuptools()
    from setuptools import setup

setup(
    setup_requires=['d2to1'],
    d2to1=True,
)



from distutils.command.build_py import build_py


def test_hook_1(config):
    print('test_hook_1')


def test_hook_2(config):
    print('test_hook_2')


class test_command(build_py):
    command_name = 'build_py'

    def run(self):
        print('Running custom build_py command.')
        return build_py.run(self)


def test_pre_hook(cmdobj):
    print('build_ext pre-hook')


def test_post_hook(cmdobj):
    print('build_ext post-hook')



#!/usr/bin/env python
try:
    from setuptools import setup
except ImportError:
    from distribute_setup import use_setuptools
    use_setuptools()
    from setuptools import setup

# d2to1 basically installs itself!  See setup.cfg for the project metadata.
from d2to1.util import cfg_to_args


setup(**cfg_to_args())



"""zest.releaser entry points to support projects using distutils2-like
setup.cfg files.  The only actual functionality this adds is to update the
version option in a setup.cfg file, if it exists.  If setup.cfg does not exist,
or does not contain a version option, then this does nothing.

TODO: d2to1 theoretically supports using a different filename for setup.cfg;
this does not support that.  We could hack in support, though I'm not sure how
useful the original functionality is to begin with (and it might be removed) so
we ignore that for now.

TODO: There exists a proposal
(http://mail.python.org/pipermail/distutils-sig/2011-March/017628.html) to add
a 'version-from-file' option (or something of the like) to distutils2; if this
is added then support for it should be included here as well.
"""


import logging
import os

from .extern.six import moves as m
ConfigParser = m.configparser.ConfigParser


logger = logging.getLogger(__name__)


def update_setupcfg_version(filename, version):
    """Opens the given setup.cfg file, locates the version option in the
    [metadata] section, updates it to the new version.
    """

    setup_cfg = open(filename).readlines()
    current_section = None
    updated = False

    for idx, line in enumerate(setup_cfg):
        m = ConfigParser.SECTCRE.match(line)
        if m:
            if current_section == 'metadata':
                # We already parsed the entire metadata section without finding
                # a version line, and are now moving into a new section
                break
            current_section = m.group('header')
            continue

        if '=' not in line:
            continue

        opt, val = line.split('=', 1)
        opt, val = opt.strip(), val.strip()
        if current_section == 'metadata' and opt == 'version':
            setup_cfg[idx] = 'version = %s\n' % version
            updated = True
            break

    if updated:
        open(filename, 'w').writelines(setup_cfg)
        logger.info("Set %s's version to %r" % (os.path.basename(filename),
                                                version))


def prereleaser_middle(data):
    filename = os.path.join(data['workingdir'], 'setup.cfg')
    if os.path.exists(filename):
        update_setupcfg_version(filename, data['new_version'])


def postreleaser_middle(data):
    filename = os.path.join(data['workingdir'], 'setup.cfg')
    if os.path.exists(filename):
        update_setupcfg_version(filename, data['dev_version'])



from . import D2to1TestCase


class TestCommands(D2to1TestCase):
    def test_custom_build_py_command(self):
        """
        Test that a custom subclass of the build_py command runs when listed in
        the commands [global] option, rather than the normal build command.
        """

        stdout, _, return_code = self.run_setup('build_py')
        assert 'Running custom build_py command.' in stdout
        assert return_code == 0
